#! /usr/bin/env python
'''\
Show differences between two configuration files (config1 - config2)

$Id$
'''
from __future__ import print_function

import sys
import io
import argparse
import package_info
import re
from feedback import die

from configobj import Section
from expconfig import ConfigObj


# creates config object cnew which only contains elements from c1 missing in c2 
def createDiffConfig(c1,c2,cnew):
    createConfigDiffWithAllSections(c1, c2, cnew)
    cleanConfig(cnew)

# recursively saves differences between two config objects c1 and c2 in cnew, 
# keeps all subsections of c1 in cnew
def createConfigDiffWithAllSections(c1, c2, cnew):
    depth = c1.depth
    if depth == 0:
        main = cnew
    else:
        main = cnew.main
    for k in c1:
        if type(c1[k]) is Section:
            cnew[k] = Section(cnew, depth+1, main)
            if k not in c2:
                cnew[k] = c1[k]
            else:
                createConfigDiffWithAllSections(c1[k],c2[k], cnew[k])
        else:
            if (k not in c2) or c2[k]!=c1[k]:
                cnew[k] = c1[k]
                

# recursively cleans empty subsections of config object, returns True if config is empty
def cleanConfig(config):
    empty = True
    for k in config.keys():
        if type(config[k]) is not Section:
            empty = False
        else:
            if cleanConfig(config[k]):
                del config[k]
            else:
                empty = False
    return empty

        
#
# Main routine
#

# Check command line

command_line = argparse.ArgumentParser(description=__doc__.split('\n', 1)[0])
# TODO: print differences option,  ...
command_line.add_argument('--indent-string', default='  ', help='set indent string [%(default)s]')
command_line.add_argument('--inline-comments' , '-c', action='store_true',
                          help='compact white space before inline comments'
                               ' (BETA)')
command_line.add_argument('--trailing-space' , '-t', action='store_true',
                          help='remove white space at end of lines')
command_line.add_argument('config1', help='original configuration file name of minuend')
command_line.add_argument('config2', help='original configuration file name of subtrahend')
command_line.add_argument('-V', '--version', action='version',
                          version=package_info.version)
args = command_line.parse_args()


# File handling

try:
    config_file1 = args.config1
    config_file2 = args.config2
    config_data1 = ConfigObj(config_file1, 
                interpolation=False, file_error=True)
    config_data2 = ConfigObj(config_file2, 
                interpolation=False, file_error=True)
except IOError as error:
    die(error.message)

# Create config with elements which are only in c1:

shared_config_data = ConfigObj(indent_type=args.indent_string)
createDiffConfig(config_data1,config_data2,shared_config_data)

# Ready to roll out

lines = io.BytesIO()
shared_config_data.write(lines)

lines.seek(0)
for line in io.TextIOWrapper(lines):
    if args.inline_comments: line = re.sub(r' = (.*?)  #', r' = \1 #', line)
    if args.trailing_space:
        print(line.rstrip())
    else:
        print(line, end='')
